(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     18362,        631]
NotebookOptionsPosition[     16417,        564]
NotebookOutlinePosition[     16964,        586]
CellTagsIndexPosition[     16921,        583]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["\<\

(The functions used in this notebook require the C++ NCGB module.)\
\>", "Text",
 Editable->False,
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{"<<", "NC`"}], "\n", 
   RowBox[{"<<", "NCGB`"}]}]}]], "Input",
 CellChangeTimes->{{3.516626328725029*^9, 3.516626341227373*^9}}],

Cell[BoxData["\<\"You are using the version of NCAlgebra which is found in:\"\
\>"], "Print",
 CellChangeTimes->{3.516626404515161*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"  \"\>", "\[InvisibleSpace]", "\<\"/Users/mauricio/NC\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["  ", "/Users/mauricio/NC", "."],
  Editable->False]], "Print",
 CellChangeTimes->{3.51662640451792*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"You can now use \\\"<< NCAlgebra`\\\" to load NCAlgebra or \
\"\>", "\[InvisibleSpace]", "\<\"\\\"<< NCGB`\\\" to load NCGB.\"\>"}],
  SequenceForm[
  "You can now use \"<< NCAlgebra`\" to load NCAlgebra or ", 
   "\"<< NCGB`\" to load NCGB."],
  Editable->False]], "Print",
 CellChangeTimes->{3.516626404520475*^9}],

Cell[BoxData["\<\"You have already loaded NCGB.m\"\>"], "Print",
 CellChangeTimes->{3.5166264045229*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\

       The Partially Prescribed Inverse Problem
\
\>", "H2",
 Editable->False,
 CellFrame->{{0, 0}, {2, 0}},
 TextAlignment->Center,
 TextJustification->0.5],

Cell["\<\

This is a type of problem known as a matrix completion problem. This \
particular one was  suggested by Hugo Woerdeman.   We are grateful to him for \
discussions.
\
\>", "Text",
 Editable->False],

Cell[TextData[StyleBox["Problem:\n     \tGiven matrices a, b, c,  and  d,    \
we  wish to determine under what conditions there exists matrices  x,  y,  z, \
and w such that the block matrices",
 FontWeight->"Bold"]], "Text"],

Cell[TextData[{
 "     \n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", GridBox[{
      {"a", "x"},
      {"y", "b"}
     }], ")"}], TraditionalForm]]],
 "                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", GridBox[{
      {"w", "c"},
      {"d", "z"}
     }], ")"}], TraditionalForm]]]
}], "Text",
 Editable->False,
 TextAlignment->Center],

Cell[TextData[{
 "\n",
 StyleBox["are inverses of each other.     Also, we wish to find formulas for \
x,  y, z,  and  w.",
  FontWeight->"Bold"],
 "\n\n    This problem was solved in a paper by  W.W. Barrett, C.R. Johnson, \
M. E. Lundquist  and H. Woerderman  ",
 StyleBox["[BJLW]",
  FontWeight->"Bold"],
 " where  they showed it splits into several cases depending upon which of a, \
b, c and d are invertible. In our example, ",
 StyleBox["we assume that a, b, c and d  are invertible",
  FontWeight->"Bold"],
 " and discover the result which they obtain in this case."
}], "Text",
 CellFrame->{{0, 0}, {2, 0}}],

Cell["\<\
First we set all of our variables to be noncommutative and set up the \
relations which make matrices a, b, c, and d invertible.  (Inverses in this \
particular problem are taken to be two sided.)   Strong invertibility \
relations help when one is trying to get an idea of the solution of the \
problem.
\
\>", "Text",
 Editable->False,
 CellFrame->True,
 Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\n", "             ", 
  RowBox[{
   RowBox[{"SetNonCommutative", "[", 
    RowBox[{
    "a", ",", "b", ",", "c", ",", "d", ",", "w", ",", "x", ",", "y", ",", "z",
      ",", " ", 
     RowBox[{"Inv", "[", "a", "]"}], ",", "  ", 
     RowBox[{"Inv", "[", "b", "]"}], ",", " ", 
     RowBox[{"Inv", "[", "c", "]"}], " ", ",", 
     RowBox[{"Inv", "[", "d", "]"}]}], "  ", "]"}], "\n"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "False", ",", "False", ",", "False", ",", "False", ",", "False", ",", 
   "False", ",", "False", ",", "False", ",", "False", ",", "False", ",", 
   "False", ",", "False"}], "}"}]], "Output",
 CellChangeTimes->{3.516626353719666*^9, 3.5166264090555973`*^9}]
}, Open  ]],

Cell[TextData[{
 "Then we define the relations we are interested in.   The  two relations ",
 StyleBox["oneway, otherway",
  FontWeight->"Bold"],
 "  set our block matrices as inverses of each other.   The relations",
 StyleBox[" inverses",
  FontWeight->"Bold"],
 "\ninvoke the assumption that ",
 StyleBox["a,b,c,",
  FontWeight->"Bold"],
 " and ",
 StyleBox["d",
  FontWeight->"Bold"],
 " are invertible by defining their inverses."
}], "Text",
 CellFrame->True,
 Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"first", " ", "=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "x"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", "b"}], "}"}]}], "}"}]}], "\n", 
 RowBox[{"second", " ", "=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"w", ",", "c"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"d", ",", "z"}], "}"}]}], "}"}]}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "x"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "b"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.516626355728785*^9, 3.516626410452258*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"w", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"d", ",", "z"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.516626355728785*^9, 3.5166264104590178`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{"oneway", " ", "=", 
    RowBox[{
     RowBox[{"MatMult", "[", 
      RowBox[{"first", ",", "second"}], "]"}], " ", "-", " ", 
     RowBox[{"IdentityMatrix", "[", "2", "]"}]}]}], "\n", 
   RowBox[{"otherway", " ", "=", "  ", 
    RowBox[{
     RowBox[{"MatMult", "[", 
      RowBox[{"second", ",", "first"}], "]"}], "-", "  ", 
     RowBox[{"IdentityMatrix", "[", "2", "]"}]}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"a", "**", "w"}], "+", 
      RowBox[{"x", "**", "d"}]}], ",", 
     RowBox[{
      RowBox[{"a", "**", "c"}], "+", 
      RowBox[{"x", "**", "z"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"b", "**", "d"}], "+", 
      RowBox[{"y", "**", "w"}]}], ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"b", "**", "z"}], "+", 
      RowBox[{"y", "**", "c"}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.516626357159501*^9, 3.516626411549324*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"c", "**", "y"}], "+", 
      RowBox[{"w", "**", "a"}]}], ",", 
     RowBox[{
      RowBox[{"c", "**", "b"}], "+", 
      RowBox[{"w", "**", "x"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "**", "a"}], "+", 
      RowBox[{"z", "**", "y"}]}], ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"d", "**", "x"}], "+", 
      RowBox[{"z", "**", "b"}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.516626357159501*^9, 3.516626411553116*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{
    RowBox[{"inverses", " ", "=", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{"a", " ", "**", " ", 
         RowBox[{"Inv", "[", "a", "]"}]}]}], ",", " ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"Inv", "[", "a", "]"}], " ", "**", " ", "a"}]}], ",", "\n", 
       "    ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{"b", " ", "**", " ", 
         RowBox[{"Inv", "[", "b", "]"}]}]}], ",", " ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"Inv", "[", "b", "]"}], " ", "**", " ", "b"}]}], ",", "\n", 
       "    ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{"c", " ", "**", " ", 
         RowBox[{"Inv", "[", "c", "]"}]}]}], ",", " ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"Inv", "[", "c", "]"}], " ", "**", " ", "c"}]}], ",", "\n", 
       "     ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{"d", " ", "**", " ", 
         RowBox[{"Inv", "[", "d", "]"}]}]}], ",", " ", 
       RowBox[{
        RowBox[{"-", "1"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"Inv", "[", "d", "]"}], " ", "**", " ", "d"}]}]}], " ", "\n",
       "\t", "}"}]}], "\n"}], "\n", 
   RowBox[{"allRelations", " ", "=", 
    RowBox[{"Join", "[", " ", 
     RowBox[{
      RowBox[{"Flatten", "[", 
       RowBox[{"{", " ", 
        RowBox[{"oneway", ",", " ", "otherway"}], "  ", "}"}], "]"}], ",", 
      "inverses"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"a", "**", 
     RowBox[{"Inv", "[", "a", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"Inv", "[", "a", "]"}], "**", "a"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"b", "**", 
     RowBox[{"Inv", "[", "b", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"Inv", "[", "b", "]"}], "**", "b"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"c", "**", 
     RowBox[{"Inv", "[", "c", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"Inv", "[", "c", "]"}], "**", "c"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"d", "**", 
     RowBox[{"Inv", "[", "d", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"Inv", "[", "d", "]"}], "**", "d"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.516626358372363*^9, 3.516626412414721*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"a", "**", "w"}], "+", 
    RowBox[{"x", "**", "d"}]}], ",", 
   RowBox[{
    RowBox[{"a", "**", "c"}], "+", 
    RowBox[{"x", "**", "z"}]}], ",", 
   RowBox[{
    RowBox[{"b", "**", "d"}], "+", 
    RowBox[{"y", "**", "w"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"b", "**", "z"}], "+", 
    RowBox[{"y", "**", "c"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"c", "**", "y"}], "+", 
    RowBox[{"w", "**", "a"}]}], ",", 
   RowBox[{
    RowBox[{"c", "**", "b"}], "+", 
    RowBox[{"w", "**", "x"}]}], ",", 
   RowBox[{
    RowBox[{"d", "**", "a"}], "+", 
    RowBox[{"z", "**", "y"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"d", "**", "x"}], "+", 
    RowBox[{"z", "**", "b"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"a", "**", 
     RowBox[{"Inv", "[", "a", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"Inv", "[", "a", "]"}], "**", "a"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"b", "**", 
     RowBox[{"Inv", "[", "b", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"Inv", "[", "b", "]"}], "**", "b"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"c", "**", 
     RowBox[{"Inv", "[", "c", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"Inv", "[", "c", "]"}], "**", "c"}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"d", "**", 
     RowBox[{"Inv", "[", "d", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"Inv", "[", "d", "]"}], "**", "d"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.516626358372363*^9, 3.516626412420635*^9}]
}, Open  ]],

Cell["\<\
Specify to NCAlgebra which variables are known and which are unknown. To GB \
fans this sets the monomial order indicated around the middle of the first \
page of the output. 
\
\>", "Text",
 CellFrame->True,
 Background->GrayLevel[0.849989]],

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{
    RowBox[{"SetKnowns", "[", 
     RowBox[{"a", ",", 
      RowBox[{"Inv", "[", "a", "]"}], ",", "b", ",", 
      RowBox[{"Inv", "[", "b", "]"}], ",", "c", ",", 
      RowBox[{"Inv", "[", "c", "]"}], ",", "d", ",", 
      RowBox[{"Inv", "[", "d", "]"}]}], "]"}], "\n"}], "\n", 
   RowBox[{
    RowBox[{"SetUnknowns", "[", 
     RowBox[{
      RowBox[{"{", "z", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y", ",", "w"}], "}"}]}], "]"}], "\n"}]}]}]], "Input"],

Cell["\<\
Tell NCAlgebra to solve for our unknown variables
\
\>", "Text",
 Editable->False,
 CellFrame->True,
 Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\n", "  ", "\n", 
  RowBox[{
   RowBox[{
    RowBox[{"answer", "=", 
     RowBox[{"NCProcess", "[", 
      RowBox[{"allRelations", ",", "4", ",", " ", "\"\<DemoGBMA\>\""}], " ", 
      "]"}]}], ";"}], "\n", "\n"}]}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Outputting results to the stream \"\>", "\[InvisibleSpace]", 
   InterpretationBox[
    StyleBox[
     RowBox[{"OutputStream", "[", 
      RowBox[{"\"\<DemoGBMA.tex\>\"", ",", " ", "44"}], "]"}],
     ShowStringCharacters->True,
     NumberMarks->True],
    Format[
     OutputStream["DemoGBMA.tex", 44], InputForm],
    AutoDelete->True,
    Editable->True]}],
  SequenceForm["Outputting results to the stream ", 
   Format[
    OutputStream["DemoGBMA.tex", 44], InputForm]],
  Editable->False]], "Print",
 CellChangeTimes->{3.5166263679611597`*^9, 3.51662641844002*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Done outputting results to the stream \"\>", 
   "\[InvisibleSpace]", 
   InterpretationBox[
    StyleBox[
     RowBox[{"OutputStream", "[", 
      RowBox[{"\"\<DemoGBMA.tex\>\"", ",", " ", "44"}], "]"}],
     ShowStringCharacters->True,
     NumberMarks->True],
    Format[
     OutputStream["DemoGBMA.tex", 44], InputForm],
    AutoDelete->True,
    Editable->True]}],
  SequenceForm["Done outputting results to the stream ", 
   Format[
    OutputStream["DemoGBMA.tex", 44], InputForm]],
  Editable->False]], "Print",
 CellChangeTimes->{3.5166263679611597`*^9, 3.516626418597641*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"> Running '\"\>", 
   "\[InvisibleSpace]", "\<\"pdflatex -output-directory=.  DemoGBMA.tex 1> \\\
\"/tmp/mNCRun.out\\\" 2> \\\"/tmp/mNCRun.err\\\"\"\>", 
   "\[InvisibleSpace]", "\<\"'...\"\>"}],
  SequenceForm[
  "> Running '", 
   "pdflatex -output-directory=.  DemoGBMA.tex 1> \"/tmp/mNCRun.out\" 2> \
\"/tmp/mNCRun.err\"", "'..."],
  Editable->False]], "Print",
 CellChangeTimes->{3.5166263679611597`*^9, 3.51662641860078*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"> Running '\"\>", 
   "\[InvisibleSpace]", "\<\"open  DemoGBMA.pdf 1> \\\"/tmp/mNCRun.out\\\" 2> \
\\\"/tmp/mNCRun.err\\\"\"\>", "\[InvisibleSpace]", "\<\"'...\"\>"}],
  SequenceForm[
  "> Running '", 
   "open  DemoGBMA.pdf 1> \"/tmp/mNCRun.out\" 2> \"/tmp/mNCRun.err\"", "'..."],
  Editable->False]], "Print",
 CellChangeTimes->{3.5166263679611597`*^9, 3.516626418736718*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Note: Your TeX and Dvi files are located in the directory \
\"\>", "\[InvisibleSpace]", "\<\"/Users/mauricio\"\>"}],
  SequenceForm[
  "Note: Your TeX and Dvi files are located in the directory ", 
   "/Users/mauricio"],
  Editable->False]], "Print",
 CellChangeTimes->{3.5166263679611597`*^9, 3.516626418844254*^9}]
}, Open  ]],

Cell["\<\

The spreadsheet which appears shows that, if a, b, c and d are invertible, \
then one can find x, y, z and w such that the matrices above are inverses of \
each other if and only if z b z = z + d a c.

The spreadsheet also gives formulas for x, y and w in terms of z.
\
\>", "Text",
 Editable->False]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
CellGrouping->Manual,
WindowSize->{895, 655},
WindowMargins->{{141, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"Magnification"->1,
"PaperOrientation"->"Portrait",
"PaperSize"->{612, 792}},
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 6, \
2010)",
StyleDefinitions->"HTML.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 124, 5, 45, "Text"],
Cell[CellGroupData[{
Cell[706, 29, 181, 5, 55, "Input"],
Cell[890, 36, 135, 2, 20, "Print"],
Cell[1028, 40, 268, 6, 20, "Print"],
Cell[1299, 48, 365, 8, 20, "Print"],
Cell[1667, 58, 104, 1, 20, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1808, 64, 168, 8, 109, "H2"],
Cell[1979, 74, 207, 7, 56, "Text"],
Cell[2189, 83, 226, 3, 41, "Text"],
Cell[2418, 88, 359, 17, 60, "Text"],
Cell[2780, 107, 616, 15, 111, "Text"],
Cell[3399, 124, 399, 10, 72, "Text"],
Cell[CellGroupData[{
Cell[3823, 138, 421, 10, 55, "Input"],
Cell[4247, 150, 298, 6, 24, "Output"]
}, Open  ]],
Cell[4560, 159, 499, 16, 72, "Text"],
Cell[CellGroupData[{
Cell[5084, 179, 411, 14, 40, "Input"],
Cell[5498, 195, 235, 7, 24, "Output"],
Cell[5736, 204, 237, 7, 24, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6010, 216, 445, 12, 55, "Input"],
Cell[6458, 230, 627, 21, 24, "Output"],
Cell[7088, 253, 627, 21, 24, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7752, 279, 1708, 49, 130, "Input"],
Cell[9463, 330, 1021, 35, 40, "Output"],
Cell[10487, 367, 1829, 63, 55, "Output"]
}, Open  ]],
Cell[12331, 433, 252, 7, 72, "Text"],
Cell[12586, 442, 527, 15, 85, "Input"],
Cell[13116, 459, 144, 6, 57, "Text"],
Cell[CellGroupData[{
Cell[13285, 469, 250, 7, 85, "Input"],
Cell[13538, 478, 620, 17, 20, "Print"],
Cell[14161, 497, 635, 18, 20, "Print"],
Cell[14799, 517, 479, 11, 20, "Print"],
Cell[15281, 530, 426, 9, 20, "Print"],
Cell[15710, 541, 365, 8, 20, "Print"]
}, Open  ]],
Cell[16090, 552, 311, 9, 101, "Text"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)

